@@ -1,9 +1,9 @@
 /*
  *  defDirect.c
- *  $Id: defDirect.c,v 4.15 2007/07/04 20:51:11 bkorb Exp $
+ *  $Id: defDirect.c,v 4.21 2008/01/23 00:36:00 bkorb Exp $
  *
- *  Time-stamp:        "2007-07-04 11:15:44 bkorb"
- *  Last Committed:    $Date: 2007/07/04 20:51:11 $
+ *  Time-stamp:        "2007-11-17 09:46:24 bkorb"
+ *  Last Committed:    $Date: 2008/01/23 00:36:00 $
  *
  *  This module processes definition file directives.
  *
@@ -15,7 +15,8 @@
  *    trail='\+\+\+ End of Directives'
  *
  *  This file is part of AutoGen.
- *  AutoGen copyright (c) 1992-2007 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -100,13 +101,15 @@ processDirective( char* pzScan )
     /*
      *  Find the start of the directive name
      */
-    while (isspace(*pzScan)) pzScan++;
+    while (IS_WHITESPACE_CHAR(*pzScan)) pzScan++;
     pzDir = pzScan;
 
     /*
-     *  Find the *END* of the directive name
+     *  Find the *END* of the directive name.  If no name, bail out.
      */
-    while (ISNAMECHAR( *pzScan )) pzScan++;
+    while (IS_ALPHABETIC_CHAR(*pzScan)) pzScan++;
+    if (pzDir == pzScan)
+        return pzEnd;
 
     /*
      *  IF there is anything that follows the name, ...
@@ -116,7 +119,7 @@ processDirective( char* pzScan )
          *  IF something funny immediately follows the directive name,
          *  THEN we will ignore it completely.
          */
-        if (! isspace( *pzScan ))
+        if (! IS_WHITESPACE_CHAR(*pzScan))
             return pzEnd;
 
         /*
@@ -124,7 +127,7 @@ processDirective( char* pzScan )
          *  and find the start of anything else.
          */
         *pzScan++ = NUL;
-        while (isspace(*pzScan)) pzScan++;
+        while (IS_WHITESPACE_CHAR(*pzScan)) pzScan++;
     }
 
     /*
@@ -132,7 +135,7 @@ processDirective( char* pzScan )
      */
     {
         char* pz = pzScan + strlen( pzScan );
-        while ((pz > pzScan) && isspace( pz[-1] )) pz--;
+        while ((pz > pzScan) && IS_WHITESPACE_CHAR(pz[-1])) pz--;
         *pz = NUL;
     }
 
@@ -155,7 +158,7 @@ findDirective( char* pzDirName )
     do  {
         if (  (strneqvcmp( pzDirName, pTbl[res].pzDirName,
                            (int)pTbl[res].nameSize ) == 0)
-           && (  isspace( pzDirName[ pTbl[res].nameSize ])
+           && (  IS_WHITESPACE_CHAR(pzDirName[ pTbl[res].nameSize ])
               || (pzDirName[ pTbl[res].nameSize ] == NUL) )  )
             return res;
 
@@ -213,7 +216,7 @@ skipToEndif( char* pzStart )
             pzScan = pz + STRSIZE( zCheckList );
         }
 
-        while (isspace( *pzScan )) pzScan++;
+        while (IS_WHITESPACE_CHAR(*pzScan)) pzScan++;
 
         switch (findDirective( pzScan )) {
         case DIR_ENDIF:
@@ -276,7 +279,7 @@ skipToEndmac( char* pzStart )
             pzScan = pz + STRSIZE( zCheckList );
         }
 
-        while (isspace( *pzScan )) pzScan++;
+        while (IS_WHITESPACE_CHAR(*pzScan)) pzScan++;
 
         if (findDirective( pzScan ) == DIR_ENDMAC) {
             /*
@@ -328,7 +331,7 @@ skipToElseEnd( char* pzStart )
             pzScan = pz + STRSIZE( zCheckList );
         }
 
-        while (isspace( *pzScan )) pzScan++;
+        while (IS_WHITESPACE_CHAR(*pzScan)) pzScan++;
 
         switch (findDirective( pzScan )) {
         case DIR_ELSE:
@@ -422,16 +425,10 @@ check_assert_str( char const* pz, char const* pzArg )
 {
     static char const fmt[] = "#assert yielded \"%s\":\n\t`%s`";
 
-    while (isspace(*pz)) pz++;
+    while (IS_WHITESPACE_CHAR(*pz)) pz++;
 
-    if (isdigit(*pz)) {
-        if (atoi(pz) == 0)
-            AG_ABEND( aprf( fmt, "0 (zero)", pzArg ));
-
-    } else switch (*pz) {
-    case 'f': case 'F': case 'n': case 'N': case NUL:
+    if (IS_FALSE_TYPE_CHAR(*pz))
         AG_ABEND( aprf( fmt, pz, pzArg ));
-    }
 }
 
 static char*
@@ -491,9 +488,9 @@ doDir_define( char* pzArg, char* pzScan )
     /*
      *  Skip any #defines that do not look reasonable
      */
-    if (! isalpha( *pzArg ))
+    if (! IS_VAR_FIRST_CHAR( *pzArg ))
         return pzScan;
-    while (ISNAMECHAR( *pzArg )) pzArg++;
+    while (IS_VARIABLE_NAME_CHAR( *pzArg )) pzArg++;
 
     /*
      *  IF this is a macro definition (rather than a value def),
@@ -509,9 +506,9 @@ doDir_define( char* pzArg, char* pzScan )
      *       Therefore, move the name back over the "#define"
      *       directive itself, giving us the space needed.
      */
-    if (! isspace( *pzArg )) {
+    if (! IS_WHITESPACE_CHAR(*pzArg)) {
         char* pzS = pzName;
-        char* pzD = --pzName;
+        char* pzD = (pzName -= 6);
 
         *pzArg = NUL;
         while ((*(pzD++) = *(pzS++)) != NUL)   ;
@@ -526,12 +523,12 @@ doDir_define( char* pzArg, char* pzScan )
          */
         char*  pz = pzArg+1;
         *pzArg++ = '=';
-        while (isspace( *pz )) pz++;
+        while (IS_WHITESPACE_CHAR(*pz)) pz++;
 
         for (;;) {
             if ((*pzArg++ = *pz++) == NUL)
                 break;
-            if (! ISNAMECHAR( *pz )) {
+            if (! IS_UNQUOTABLE_CHAR(*pz)) {
                 *pzArg = NUL;
                 break;
             }
@@ -843,8 +840,8 @@ doDir_line( char* pzArg, char* pzScan )
      *
      *  Start by scanning up to and extracting the line number.
      */
-    while (isspace( *pzArg )) pzArg++;
-    if (! isdigit( *pzArg ))
+    while (IS_WHITESPACE_CHAR(*pzArg)) pzArg++;
+    if (! IS_DEC_DIGIT_CHAR(*pzArg))
         return pzScan;
 
     pCurCtx->lineNo = strtol( pzArg, &pzArg, 0 );
@@ -853,7 +850,7 @@ doDir_line( char* pzArg, char* pzScan )
      *  Now extract the quoted file name string.
      *  We dup the string so it won't disappear on us.
      */
-    while (isspace( *pzArg )) pzArg++;
+    while (IS_WHITESPACE_CHAR(*pzArg)) pzArg++;
     if (*(pzArg++) != '"')
         return pzScan;
     {
@@ -977,9 +974,6 @@ doDir_shell( char* pzArg, char* pzScan )
      */
     pCurCtx->pzScan  = pzScan;
 
-    if (pzShellProgram == NULL)
-        pzShellProgram = getDefine( zShellEnv, AG_TRUE );
-
     /*
      *  Run the shell command.  The output text becomes the
      *  "file text" that is used for more definitions.
