@@ -1,13 +1,14 @@
 /*
- *  $Id: defFind.c,v 4.15 2007/08/05 17:09:50 bkorb Exp $
+ *  $Id: defFind.c,v 4.18 2008/01/23 00:36:00 bkorb Exp $
  *
- *  Time-stamp:        "2007-07-04 11:15:50 bkorb"
- *  Last Committed:    $Date: 2007/08/05 17:09:50 $
+ *  Time-stamp:        "2007-11-12 20:44:29 bkorb"
+ *  Last Committed:    $Date: 2008/01/23 00:36:00 $
  *
  *  This module locates definitions.
  *
  *  This file is part of AutoGen.
- *  AutoGen copyright (c) 1992-2007 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -75,7 +76,7 @@ findEntryByIndex( tDefEntry* pE, char* pzScan )
      *  '[$]' means the last entry of whatever index number
      */
     if (*pzScan == '$') {
-        while (isspace( *++pzScan )) ;
+        while (IS_WHITESPACE_CHAR(*++pzScan )) ;
         if (*pzScan != ']')
             return NULL;
 
@@ -88,14 +89,14 @@ findEntryByIndex( tDefEntry* pE, char* pzScan )
     /*
      *  '[nn]' means the specified index number
      */
-    if (isdigit( *pzScan )) {
+    if (IS_DEC_DIGIT_CHAR(*pzScan)) {
         char* pz;
         idx = strtol( pzScan, &pz, 0 );
 
         /*
          *  Skip over any trailing space and make sure we have a closer
          */
-        while (isspace( *pz )) pz++;
+        while (IS_WHITESPACE_CHAR(*pz )) pz++;
         if (*pz != ']')
             return NULL;
     }
@@ -107,10 +108,10 @@ findEntryByIndex( tDefEntry* pE, char* pzScan )
         char* pzDef = pzScan;
         char const* pzVal;
 
-        if (! isalpha( *pzScan ))
+        if (! IS_VAR_FIRST_CHAR(*pzScan))
             return NULL;
 
-        while (ISNAMECHAR( *pzScan )) pzScan++;
+        while (IS_VALUE_NAME_CHAR(*pzScan)) pzScan++;
 
         /*
          *  Temporarily remove the character under *pzScan and
@@ -126,7 +127,7 @@ findEntryByIndex( tDefEntry* pE, char* pzScan )
         /*
          *  Skip over any trailing space and make sure we have a closer
          */
-        while (isspace( *pzScan )) pzScan++;
+        while (IS_WHITESPACE_CHAR(*pzScan )) pzScan++;
         if (*pzScan != ']')
             return NULL;
 
@@ -222,7 +223,7 @@ canonicalizeName( char* pzD, char const* pzS, int srcLen )
      *  Before anything, skip a leading '.' as a special hack to force
      *  a current context lookup.
      */
-    while (isspace( *pzS )) {
+    while (IS_WHITESPACE_CHAR(*pzS )) {
         if (--srcLen <= 0) {
             pzS = zNil;
             break;
@@ -241,7 +242,7 @@ canonicalizeName( char* pzD, char const* pzS, int srcLen )
      *  but an index may also start with a number.  The full number
      *  validation will happen in findEntryByIndex().
      */
-    while (isspace( *pzS )) {
+    while (IS_WHITESPACE_CHAR(*pzS )) {
         if (--srcLen <= 0) {
             pzS = zNil;
             break;
@@ -251,7 +252,7 @@ canonicalizeName( char* pzD, char const* pzS, int srcLen )
 
     switch (state) {
     case CN_START_NAME:
-        if (! isalpha( *pzS ))
+        if (! IS_VAR_FIRST_CHAR(*pzS))
             return badName( pzDst, pzOri, stLen );
         state = CN_NAME_ENDED;  /* we found the start of our first name */
         break;  /* fall through to name/number consumption code */
@@ -296,7 +297,7 @@ canonicalizeName( char* pzD, char const* pzS, int srcLen )
          *  Numbers and #define-d names are handled at the end of the switch.
          *  '$' and ']' are handled immediately below.
          */
-        if (isalnum( *pzS ))
+        if (IS_ALPHANUMERIC_CHAR(*pzS))
             break;
 
         /*
@@ -344,15 +345,15 @@ canonicalizeName( char* pzD, char const* pzS, int srcLen )
      *  Whatever, the next token must be a name or a number.
      */
     assert((state == CN_NAME_ENDED) || (state == CN_INDEX_CLOSE));
-    assert( isalnum( *pzS ));
+    assert( IS_ALPHANUMERIC_CHAR(*pzS));
 
     /*
      *  Copy the name/number.  We already know the first character is valid.
      *  However, we must *NOT* downcase #define names...
      */
-    while (ISNAMECHAR( *pzS )) {
+    while (IS_VALUE_NAME_CHAR(*pzS)) {
         char ch = *(pzS++);
-        if ((state != CN_INDEX_CLOSE) && isupper( ch ))
+        if ((state != CN_INDEX_CLOSE) && IS_UPPER_CASE_CHAR(ch))
             *(pzD++) = tolower( ch );
 
         else switch ( ch ) { /* force the separator chars to be '_' */
@@ -472,7 +473,7 @@ defEntrySearch( char* pzName, tDefCtx* pDefCtx, ag_bool* pIsIndexed )
         /*
          *  We have to find a specific entry in a list.
          */
-        while (isspace( *++pcBrace )) ;
+        while (IS_WHITESPACE_CHAR(*++pcBrace )) ;
 
         pE = findEntryByIndex( pE, pcBrace );
         if (pE == NULL)
@@ -664,7 +665,7 @@ entryListSearch( char* pzName, tDefCtx* pDefCtx )
         /*
          *  We have to find a specific entry in a list.
          */
-        while (isspace( *++pcBrace )) ;
+        while (IS_WHITESPACE_CHAR(*++pcBrace)) ;
 
         pE = findEntryByIndex( pE, pcBrace );
         if (pE == NULL)
