@@ -1,14 +1,15 @@
 
 /*
- *  $Id: funcDef.c,v 4.20 2007/08/05 17:09:51 bkorb Exp $
+ *  $Id: funcDef.c,v 4.29 2008/01/23 00:36:01 bkorb Exp $
  *
- *  Time-stamp:        "2007-07-04 11:26:09 bkorb"
- *  Last Committed:    $Date: 2007/08/05 17:09:51 $
+ *  Time-stamp:        "2007-12-02 18:13:29 bkorb"
+ *  Last Committed:    $Date: 2008/01/23 00:36:01 $
  *
  *  This module implements the DEFINE text function.
  *
  *  This file is part of AutoGen.
- *  AutoGen copyright (c) 1992-2007 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -52,8 +53,6 @@ static void
 build_defs( int defCt, tDefList* pList );
 /* = = = END-STATIC-FORWARD = = = */
 
-tMacro* mLoad_Debug( tTemplate* pT, tMacro* pMac, tCC** ppzScan );
-
 static int
 orderDefList( const void* p1, const void* p2 )
 {
@@ -136,18 +135,18 @@ parseMacroArgs( tTemplate* pT, tMacro* pMac )
      */
     while (*pzScan != NUL) {
         ct++;
-        if (! isalpha( *pzScan )) {
+        if (! IS_VAR_FIRST_CHAR( *pzScan )) {
             fprintf( stderr, "On macro argument # %d:\n%s\n", ct, pzScan );
             AG_ABEND_IN( pT, pMac, "no macro arg name" );
         }
 
-        while (ISNAMECHAR( *pzScan ))  pzScan++;
-        while (isspace( *pzScan ))     pzScan++;
+        while (IS_VALUE_NAME_CHAR(*pzScan))  pzScan++;
+        while (IS_WHITESPACE_CHAR(*pzScan))  pzScan++;
         if (*pzScan != '=')
             continue;
-        while (isspace( *++pzScan ))   ;
+        while (IS_WHITESPACE_CHAR(*++pzScan))   ;
         pzScan = (char*)skipExpression( pzScan, strlen( pzScan ));
-        while (isspace( *pzScan ))     pzScan++;
+        while (IS_WHITESPACE_CHAR(*pzScan))     pzScan++;
     }
 
     /*
@@ -173,7 +172,7 @@ parseMacroArgs( tTemplate* pT, tMacro* pMac )
      */
     for (;; pDL++ ) {
         pDL->de.pzDefName = pzScan;
-        while (ISNAMECHAR( *pzScan ))  pzScan++;
+        while (IS_VALUE_NAME_CHAR(*pzScan))  pzScan++;
 
         switch (*pzScan) {
         case NUL:
@@ -184,7 +183,7 @@ parseMacroArgs( tTemplate* pT, tMacro* pMac )
 
         case ' ': case '\t': case '\n': case '\f':
             *(pzScan++) = NUL;
-            while (isspace( *pzScan )) pzScan++;
+            while (IS_WHITESPACE_CHAR(*pzScan)) pzScan++;
 
             /*
              *  The name was separated by space, but has no value
@@ -204,7 +203,7 @@ parseMacroArgs( tTemplate* pT, tMacro* pMac )
          *  When we arrive here, we have just clobbered a '=' char.
          *  Now we have gather up the assigned value.
          */
-        while (isspace( *pzScan ))     pzScan++;
+        while (IS_WHITESPACE_CHAR(*pzScan))     pzScan++;
         strtransform( pDL->de.pzDefName, pDL->de.pzDefName );
         pDL->pzExpr = pzScan;
         pDL->de.valType = VALTYP_TEXT;
@@ -262,14 +261,14 @@ parseMacroArgs( tTemplate* pT, tMacro* pMac )
         if (*pzScan == NUL)
             break;
 
-        if (! isspace( *pzScan ))
+        if (! IS_WHITESPACE_CHAR(*pzScan))
             AG_ABEND_IN( pT, pMac, "no space separating entries" );
 
         /*
          *  Terminate the string value and skip over any additional space
          */
         *(pzScan++) = NUL;
-        while (isspace( *pzScan ))     pzScan++;
+        while (IS_WHITESPACE_CHAR(*pzScan))     pzScan++;
     fill_in_array_continue:;
     } fill_in_array_done:;
 
@@ -337,11 +336,11 @@ prepInvokeArgs( tMacro* pMac )
      *  the arguments to the macro
      */
     else {
-        if (! isspace( *pzText ))
+        if (! IS_WHITESPACE_CHAR(*pzText))
             AG_ABEND_IN( pT, pMac,
                         "The INVOKE macro name not space separated" );
         *pzText = NUL;
-        while (isspace( *++pzText ))  ;
+        while (IS_WHITESPACE_CHAR(*++pzText))  ;
         pMac->ozText = pzText - pT->pzTemplText;
         parseMacroArgs( pT, pMac );
         pCurTemplate = pT;
@@ -349,47 +348,73 @@ prepInvokeArgs( tMacro* pMac )
 }
 
 
-#ifdef DEBUG_ENABLED
-/*=macfunc DEBUG, ifdef DEBUG_ENABLED
+/*=macfunc DEBUG
  *
- *  handler_proc:
- *  what:  Provide break point spots
+ *  handler-proc:
+ *  load-proc:
+ *
+ *  what:  Print debug message to trace output
  *  desc:
- *      By inserting [+DEBUG n+] into your template, you can set
- *      a breakpoint on the #n case element below and step through
- *      the processing of interesting parts of your template.
+ *
+ *      If the tracing level is at "debug-message" or above
+ *      (@pxref{autogen trace}), this macro prints a debug message to trace
+ *      output.  This message is not evaluated.  This macro can also be used to
+ *      set useful debugger breakpoints.  By inserting [+DEBUG n+] into your
+ *      template, you can set a debugger breakpoint on the #n case element
+ *      below (in the AutoGen source) and step through the processing of
+ *      interesting parts of your template.
+ *
+ *      To be useful, you have to have access to the source tree where autogen
+ *      was built and the template being processed.  The definitions are also
+ *      helpful, but not crucial.  Please contact the author if you think you
+ *      might actually want to use this.
 =*/
 tMacro*
 mFunc_Debug( tTemplate* pT, tMacro* pMac )
 {
     static int dummy = 0;
+    char const * pz  = pT->pzTemplText + pMac->ozText;
+    int  for_index = (forInfo.fi_depth <= 0)
+        ? -1
+        : forInfo.fi_data[ forInfo.fi_depth - 1].for_index;
+
+    fprintf(pfTrace, "  --  DEBUG %s -- FOR index %d", pz, for_index);
 
-    if (OPT_VALUE_TRACE > TRACE_NOTHING)
-        fprintf( pfTrace, "  --  DEBUG %s -- FOR index %d\n",
-                 pT->pzTemplText + pMac->ozText, (forInfo.fi_depth <= 0) ? -1
-                 : forInfo.fi_data[ forInfo.fi_depth - 1].for_index );
     /*
      *  The case element values were chosen to thwart most
      *  optimizers that might be too bright for its own good.
      *  (`dummy' is write-only and could be ignored)
      */
-    switch (atoi( pT->pzTemplText + pMac->ozText )) {
+    do  {
+        if (IS_DEC_DIGIT_CHAR(*pz)) {
+            for_index = atoi(pz);
+            break;
+        }
+    } while (*(pz++) != NUL);
+
+    if (for_index < 0)
+        for_index = -1;
+
+    switch (for_index) {
+    case -1:   dummy = 'X'; break;
     case 0:    dummy = 'A'; break;
     case 1:    dummy = 'u'; break;
     case 2:    dummy = 't'; break;
-    case 4:    dummy = 'o'; break;
-    case 8:    dummy = 'G'; break;
-    case 16:   dummy = 'e'; break;
-    case 32:   dummy = 'n'; break;
-    case 64:   dummy = 'X'; break;
-    case 128:  dummy = 'Y'; break;
-    case 256:  dummy = 'Z'; break;
-    case 512:  dummy = '.'; break;
+    case 3:    dummy = 'o'; break;
+    case 4:    dummy = 'G'; break;
+    case 5:    dummy = 'e'; break;
+    case 6:    dummy = 'n'; break;
+    case 7:    dummy = 'N'; break;
+    case 8:    dummy = 'U'; break;
+    case 9:    dummy = 'T'; break;
+    case 10:   dummy = '.'; break;
     default:   dummy++;
     }
+    if (IS_GRAPHIC_CHAR(dummy))
+        fprintf(pfTrace, " (%c)", dummy);
+    putc('\n', pfTrace);
     return pMac+1;
 }
-#endif
 
 
 /*
@@ -417,7 +442,7 @@ build_defs( int defCt, tDefList* pList )
         {
             char* pz = strchr( pList->pzExpr, '\n' );
             if (pz != NULL) {
-                while (isspace( *++pz ))  ;
+                while (IS_WHITESPACE_CHAR(*++pz))  ;
                 pList->pzExpr = pz;
                 goto retryExpression;
             }
@@ -557,7 +582,7 @@ mFunc_Define( tTemplate* pT, tMacro* pMac )
 
     if (OPT_VALUE_TRACE > TRACE_NOTHING) {
         fprintf( pfTrace, zTplInvoked, pT->pzTplName, defCt );
-        if (OPT_VALUE_TRACE < TRACE_EVERYTHING)
+        if (OPT_VALUE_TRACE == TRACE_EVERYTHING)
             fprintf( pfTrace, zFileLine, pCurTemplate->pzTplFile,
                      pMac->lineNo );
     }
@@ -650,7 +675,7 @@ mFunc_Invoke( tTemplate* pT, tMacro* pMac )
          *  THEN go find the template now and bind the macro call
          *       to a particular template macro
          */
-        if (isalpha( pT->pzTemplText[ pMac->ozName ])) {
+        if (IS_VAR_FIRST_CHAR(pT->pzTemplText[ pMac->ozName ])) {
             pMac->funcCode    = FTYP_DEFINE;
             pMac->funcPrivate = (void*)findTemplate(
                 pT->pzTemplText + pMac->ozName );
@@ -689,11 +714,13 @@ mFunc_Invoke( tTemplate* pT, tMacro* pMac )
  *
  *  what:   Loads the debug function so you can set breakpoints
  *          at load time, too :-)
-=*/
+ */
 tMacro*
-mLoad_Debug( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
+mLoad_Debug(tTemplate* pT, tMacro* pMac, tCC** ppzScan)
 {
-    return mLoad_Unknown( pT, pMac, ppzScan );
+    if (OPT_VALUE_TRACE >= TRACE_DEBUG_MESSAGE)
+        return mLoad_Unknown(pT, pMac, ppzScan);
+    return mLoad_Comment(pT, pMac, ppzScan);
 }
 
 
@@ -747,10 +774,10 @@ mLoad_Define( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
         pNewT->pzTplFile  = strdup(pT->pzTplFile);
 
         pzCopy = pNewT->pzTplName = (void*)(pNewT->aMacros + macCt);
-        if (! isalpha( *pzSrc ))
+        if (! IS_VAR_FIRST_CHAR( *pzSrc ))
             AG_ABEND_IN( pT, pMac, zNameNeeded );
 
-        while (ISNAMECHAR(*pzSrc))  *(pzCopy++) = *(pzSrc++);
+        while (IS_VALUE_NAME_CHAR(*pzSrc))  *(pzCopy++) = *(pzSrc++);
     }
 
     if (OPT_VALUE_TRACE >= TRACE_BLOCK_MACROS)
@@ -818,8 +845,8 @@ mLoad_Define( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
     if (HAVE_OPT( SHOW_DEFS )) {
         tSCC zSum[] = "loaded %d macros from %s\n"
             "\tBinary template size:  0x%X\n\n";
-        fprintf( pfTrace, zSum, pNewT->macroCt, pNewT->pzTplFile,
-                 pNewT->descSize );
+        fprintf(pfTrace, zSum, pNewT->macroCt, pNewT->pzTplFile,
+                (unsigned int)pNewT->descSize);
     }
 #endif
 
