@@ -1,57 +1,36 @@
 
 /*
- *  $Id: nested.c,v 4.13 2007/01/26 19:57:31 bkorb Exp $
- *  Time-stamp:      "2007-01-26 11:04:35 bkorb"
+ *  $Id: nested.c,v 4.22 2008/01/23 00:36:04 bkorb Exp $
+ *  Time-stamp:      "2008-01-22 11:40:15 bkorb"
  *
  *   Automated Options Nested Values module.
+ *
+ *  This file is part of AutoOpts, a companion to AutoGen.
+ *  AutoOpts is free software.
+ *  AutoOpts is copyright (c) 1992-2008 by Bruce Korb - all rights reserved
+ *  AutoOpts is copyright (c) 1992-2008 by Bruce Korb - all rights reserved
+ *
+ *  AutoOpts is available under any one of two licenses.  The license
+ *  in use must be one of these two and the choice is under the control
+ *  of the user of the license.
+ *
+ *   The GNU Lesser General Public License, version 3 or later
+ *      See the files "COPYING.lgplv3" and "COPYING.gplv3"
+ *
+ *   The Modified Berkeley Software Distribution License
+ *      See the file "COPYING.mbsd"
+ *
+ *  These files have the following md5sums:
+ *
+ *  239588c55c22c60ffe159946a760a33e pkg/libopts/COPYING.gplv3
+ *  fa82ca978890795162346e661b47161a pkg/libopts/COPYING.lgplv3
+ *  66a5cedaf62c4b2637025f049f9b826f pkg/libopts/COPYING.mbsd
  */
 
-/*
- *  Automated Options copyright 1992-2006 Bruce Korb
- *
- *  Automated Options is free software.
- *  You may redistribute it and/or modify it under the terms of the
- *  GNU General Public License, as published by the Free Software
- *  Foundation; either version 2, or (at your option) any later version.
- *
- *  Automated Options is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Automated Options.  See the file "COPYING".  If not,
- *  write to:  The Free Software Foundation, Inc.,
- *             51 Franklin Street, Fifth Floor,
- *             Boston, MA  02110-1301, USA.
- *
- * As a special exception, Bruce Korb gives permission for additional
- * uses of the text contained in his release of AutoOpts.
- *
- * The exception is that, if you link the AutoOpts library with other
- * files to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the AutoOpts library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by Bruce Korb under
- * the name AutoOpts.  If you copy code from other sources under the
- * General Public License into a copy of AutoOpts, as the General Public
- * License permits, the exception does not apply to the code that you add
- * in this way.  To avoid misleading anyone as to the status of such
- * modified files, you must delete this exception notice from them.
- *
- * If you write modifications of your own for AutoOpts, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.
- */
 /* = = = START-STATIC-FORWARD = = = */
-/* static forward declarations maintained by :mkfwd */
+/* static forward declarations maintained by mk-fwd */
 static void
-removeBackslashes( char* pzSrc );
+removeLineContinue( char* pzSrc );
 
 static char const*
 scanQuotedString( char const* pzTxt );
@@ -85,27 +64,42 @@ static void
 sortNestedList( tArgList* pAL );
 /* = = = END-STATIC-FORWARD = = = */
 
-/*  removeBackslashes
+/*  removeLineContinue
  *
- *  This function assumes that all newline characters were preceeded by
- *  backslashes that need removal.
+ *  Backslashes are used for line continuations.  We keep the newline
+ *  characters, but trim out the backslash:
  */
 static void
-removeBackslashes( char* pzSrc )
+removeLineContinue( char* pzSrc )
 {
-    char* pzD = strchr(pzSrc, '\n');
+    char* pzD;
 
-    if (pzD == NULL)
-        return;
-    *--pzD = '\n';
+    do  {
+        while (*pzSrc == '\n')  pzSrc++;
+        pzD = strchr(pzSrc, '\n');
+        if (pzD == NULL)
+            return;
 
+        /*
+         *  pzD has skipped at least one non-newline character and now
+         *  points to a newline character.  It now becomes the source and
+         *  pzD goes to the previous character.
+         */
+        pzSrc = pzD--;
+        if (*pzD != '\\')
+            pzD++;
+    } while (pzD == pzSrc);
+
+    /*
+     *  Start shifting text.
+     */
     for (;;) {
         char ch = ((*pzD++) = *(pzSrc++));
         switch (ch) {
-        case '\n': *--pzD = ch; break;
         case NUL:  return;
-        default:
-            ;
+        case '\\':
+            if (*pzSrc == '\n')
+                --pzD; /* rewrite on next iteration */
         }
     }
 }
@@ -197,22 +191,16 @@ addBoolValue( void** pp, char const* pzName, size_t nameLen,
     pNV = AGALOC( sz, "option name/bool value pair" );
     if (pNV == NULL)
         return NULL;
-    while (isspace( (int)*pzValue ) && (dataLen > 0)) {
+    while (IS_WHITESPACE_CHAR(*pzValue) && (dataLen > 0)) {
         dataLen--; pzValue++;
     }
     if (dataLen == 0)
         pNV->v.boolVal = 0;
-    else if (isdigit( (int)*pzValue ))
-        pNV->v.boolVal = atoi( pzValue );
-    else switch (*pzValue) {
-    case 'f':
-    case 'F':
-    case 'n':
-    case 'N':
-        pNV->v.boolVal = 0; break;
-    default:
-        pNV->v.boolVal = 1;
-    }
+
+    else if (IS_DEC_DIGIT_CHAR(*pzValue))
+        pNV->v.boolVal = atoi(pzValue);
+
+    else pNV->v.boolVal = ! IS_FALSE_TYPE_CHAR(*pzValue);
 
     pNV->valType = OPARG_TYPE_BOOLEAN;
     pNV->pzName = (char*)(pNV + 1);
@@ -237,16 +225,16 @@ addNumberValue( void** pp, char const* pzName, size_t nameLen,
     pNV = AGALOC( sz, "option name/bool value pair" );
     if (pNV == NULL)
         return NULL;
-    while (isspace( (int)*pzValue ) && (dataLen > 0)) {
+    while (IS_WHITESPACE_CHAR(*pzValue) && (dataLen > 0)) {
         dataLen--; pzValue++;
     }
     if (dataLen == 0)
-        pNV->v.boolVal = 0;
+        pNV->v.longVal = 0;
     else
-        pNV->v.boolVal = atoi( pzValue );
+        pNV->v.longVal = strtol(pzValue, 0, 0);
 
     pNV->valType = OPARG_TYPE_NUMERIC;
-    pNV->pzName = (char*)(pNV + 1);
+    pNV->pzName  = (char*)(pNV + 1);
     memcpy( pNV->pzName, pzName, nameLen );
     pNV->pzName[ nameLen ] = NUL;
     addArgListEntry( pp, pNV );
@@ -295,46 +283,39 @@ static char const*
 scanNameEntry(char const* pzName, tOptionValue* pRes)
 {
     tOptionValue* pNV;
-    char const * pzScan = pzName+1;
+    char const * pzScan = pzName+1; /* we know first char is a name char */
     char const * pzVal;
     size_t       nameLen = 1;
     size_t       dataLen = 0;
 
-    while (ISNAMECHAR( (int)*pzScan ))  { pzScan++; nameLen++; }
-
-    while (isspace( (int)*pzScan )) {
-        char ch = *(pzScan++);
-        if ((ch == '\n') || (ch == ',')) {
-            addStringValue(&(pRes->v.nestVal), pzName, nameLen, NULL,(size_t)0);
-            return pzScan - 1;
-        }
-    }
+    /*
+     *  Scan over characters that name a value.  These names may not end
+     *  with a colon, but they may contain colons.
+     */
+    while (IS_VALUE_NAME_CHAR(*pzScan))      { pzScan++; nameLen++; }
+    if (pzScan[-1] == ':')              { pzScan--; nameLen--; }
+    while (IS_HORIZ_WHITE_CHAR(*pzScan))     pzScan++;
 
+re_switch:
     switch (*pzScan) {
     case '=':
     case ':':
-        while (isspace( (int)*++pzScan ))  ;
-        switch (*pzScan) {
-        case ',':  goto comma_char;
-        case '"':
-        case '\'': goto quote_char;
-        case NUL:  goto nul_byte;
-        default:   goto default_char;
-        }
+        while (IS_HORIZ_WHITE_CHAR( (int)*++pzScan ))  ;
+        if ((*pzScan == '=') || (*pzScan == ':'))
+            goto default_char;
+        goto re_switch;
 
+    case '\n':
     case ',':
-    comma_char:
         pzScan++;
         /* FALLTHROUGH */
 
     case NUL:
-    nul_byte:
         addStringValue(&(pRes->v.nestVal), pzName, nameLen, NULL, (size_t)0);
         break;
 
     case '"':
     case '\'':
-    quote_char:
         pzVal = pzScan;
         pzScan = scanQuotedString( pzScan );
         dataLen = pzScan - pzVal;
@@ -373,7 +354,7 @@ scanNameEntry(char const* pzName, tOptionValue* pRes)
                 pNV = addStringValue( &(pRes->v.nestVal), pzName, nameLen,
                                       pzVal, dataLen );
                 if (pNV != NULL)
-                    removeBackslashes( pNV->v.strVal );
+                    removeLineContinue( pNV->v.strVal );
                 goto leave_scan_name;
             }
         }
@@ -400,7 +381,7 @@ scanXmlEntry( char const* pzName, tOptionValue* pRes )
     tOptionValue* pNewVal;
     tOptionLoadMode save_mode = option_load_mode;
 
-    if (! isalpha((int)*pzName)) {
+    if (! IS_VAR_FIRST_CHAR(*pzName)) {
         switch (*pzName) {
         default:
             pzName = NULL;
@@ -421,7 +402,8 @@ scanXmlEntry( char const* pzName, tOptionValue* pRes )
         return pzName;
     }
 
-    while (isalpha( (int)*++pzScan ))  nameLen++;
+    pzScan++;
+    while (IS_VALUE_NAME_CHAR( (int)*pzScan ))  { pzScan++; nameLen++; }
     if (nameLen > 64)
         return NULL;
     valu.valType = OPARG_TYPE_STRING;
@@ -484,7 +466,7 @@ scanXmlEntry( char const* pzName, tOptionValue* pRes )
         }
         valLen = (pzScan - pzVal);
         pzScan += nameLen + 3;
-        while (isspace(  (int)*pzScan ))  pzScan++;
+        while (IS_WHITESPACE_CHAR(*pzScan))  pzScan++;
     }
 
     switch (valu.valType) {
@@ -651,7 +633,7 @@ optionLoadNested(char const* pzTxt, char const* pzName, size_t nameLen)
         errno = EINVAL;
         return NULL;
     }
-    while (isspace( (int)*pzTxt ))  pzTxt++;
+    while (IS_WHITESPACE_CHAR(*pzTxt))  pzTxt++;
     if (*pzTxt == NUL) {
         errno = ENOENT;
         return NULL;
@@ -679,13 +661,14 @@ optionLoadNested(char const* pzTxt, char const* pzName, size_t nameLen)
      *  Scan until we hit a NUL.
      */
     do  {
-        while (isspace( (int)*pzTxt ))  pzTxt++;
-        if (isalpha( (int)*pzTxt )) {
+        while (IS_WHITESPACE_CHAR( (int)*pzTxt ))  pzTxt++;
+        if (IS_VAR_FIRST_CHAR( (int)*pzTxt )) {
             pzTxt = scanNameEntry( pzTxt, pRes );
         }
         else switch (*pzTxt) {
         case NUL: goto scan_done;
         case '<': pzTxt = scanXmlEntry( pzTxt, pRes );
+                  if (pzTxt == NULL) goto woops;
                   if (*pzTxt == ',') pzTxt++;     break;
         case '#': pzTxt = strchr( pzTxt, '\n' );  break;
         default:  goto woops;
